-- Key Symbol definitions taken from
-- /usr/include/X11/keysymdef.h

-- According to my tests with xdotool, some of the higher keysyms may not be
-- accurate. It would print the lowercase 'ň' when using 'Ncaron', for example.
-- Because of this, codepoints should be preferred over keysyms.


local function format(rawdata)
  local keysyms = {}
 
  for i, raw in ipairs(rawdata) do
    local sym = {
        code = raw[1],
        str  = raw[2],
        live = raw[3],
        dead = raw[4],
    }
    keysyms[sym.str] = sym
    keysyms[i] = sym
  end
 
  return keysyms
end


return format({
  -- Raw data format:
  -- {codepoint, utf8-string, keysym, dead_keysym}

  -- Basic Latin (ASCII)
  {0x0020, ' ', 'space'},
  {0x0021, '!', 'exclam'},
  {0x0022, '\"','quotedbl'},
  {0x0023, '#', 'numbersign'},
  {0x0024, '$', 'dollar'},
  {0x0025, '%', 'percent'},
  {0x0026, '&', 'ampersand'},
  {0x0027, '\'','apostrophe'},
  {0x0028, '(', 'parenleft'},
  {0x0029, ')', 'parenright'},
  {0x002A, '*', 'asterisk'},
  {0x002B, '+', 'plus'},
  {0x002C, ',', 'comma'},
  {0x002D, '-', 'minus'},
  {0x002E, '.', 'period'},
  {0x002F, '/', 'slash'},
  {0x0030, '0', '0'},
  {0x0031, '1', '1'},
  {0x0032, '2', '2'},
  {0x0033, '3', '3'},
  {0x0034, '4', '4'},
  {0x0035, '5', '5'},
  {0x0036, '6', '6'},
  {0x0037, '7', '7'},
  {0x0038, '8', '8'},
  {0x0039, '9', '9'},
  {0x003A, ':', 'colon'},
  {0x003B, ';', 'semicolon'},
  {0x003C, '<', 'less'},
  {0x003D, '=', 'equal'},
  {0x003E, '>', 'greater'},
  {0x003F, '?', 'question'},
  {0x0040, '@', 'at'},
  {0x0041, 'A', 'A'},
  {0x0042, 'B', 'B'},
  {0x0043, 'C', 'C'},
  {0x0044, 'D', 'D'},
  {0x0045, 'E', 'E'},
  {0x0046, 'F', 'F'},
  {0x0047, 'G', 'G'},
  {0x0048, 'H', 'H'},
  {0x0049, 'I', 'I'},
  {0x004A, 'J', 'J'},
  {0x004B, 'K', 'K'},
  {0x004C, 'L', 'L'},
  {0x004D, 'M', 'M'},
  {0x004E, 'N', 'N'},
  {0x004F, 'O', 'O'},
  {0x0050, 'P', 'P'},
  {0x0051, 'Q', 'Q'},
  {0x0052, 'R', 'R'},
  {0x0053, 'S', 'S'},
  {0x0054, 'T', 'T'},
  {0x0055, 'U', 'U'},
  {0x0056, 'V', 'V'},
  {0x0057, 'W', 'W'},
  {0x0058, 'X', 'X'},
  {0x0059, 'Y', 'Y'},
  {0x005A, 'Z', 'Z'},
  {0x005B, '[', 'bracketleft'},
  {0x005C, '\\','backslash'},
  {0x005D, ']', 'bracketright'},
  {0x005E, '^', 'asciicircum', 'dead_circumflex'},
  {0x005F, '_', 'underscore'},
  {0x0060, '`', 'grave', 'dead_grave'},
  {0x0061, 'a', 'a'},
  {0x0062, 'b', 'b'},
  {0x0063, 'c', 'c'},
  {0x0064, 'd', 'd'},
  {0x0065, 'e', 'e'},
  {0x0066, 'f', 'f'},
  {0x0067, 'g', 'g'},
  {0x0068, 'h', 'h'},
  {0x0069, 'i', 'i'},
  {0x006A, 'j', 'j'},
  {0x006B, 'k', 'k'},
  {0x006C, 'l', 'l'},
  {0x006D, 'm', 'm'},
  {0x006E, 'n', 'n'},
  {0x006F, 'o', 'o'},
  {0x0070, 'p', 'p'},
  {0x0071, 'q', 'q'},
  {0x0072, 'r', 'r'},
  {0x0073, 's', 's'},
  {0x0074, 't', 't'},
  {0x0075, 'u', 'u'},
  {0x0076, 'v', 'v'},
  {0x0077, 'w', 'w'},
  {0x0078, 'x', 'x'},
  {0x0079, 'y', 'y'},
  {0x007A, 'z', 'z'},
  {0x007B, '{', 'braceleft'},
  {0x007C, '|', 'bar'},
  {0x007D, '}', 'braceright'},
  {0x007E, '~', 'asciitilde', 'dead_tilde'},

  -- Latin-1 Supplement
    -- {0x00A0, , 'nobreakspace'},
  {0x00A1, '¡', 'exclamdown'},
  {0x00A2, '¢', 'cent'},
  {0x00A3, '£', 'sterling'},
  {0x00A4, '¤', 'currency', 'dead_currency'},
  {0x00A5, '¥', 'yen'},
  {0x00A6, '¦', 'brokenbar'},
  {0x00A7, '§', 'section'},
  {0x00A8, '¨', 'diaeresis', 'dead_diaeresis'},
  {0x00A9, '©', 'copyright'},
  {0x00AA, 'ª', 'ordfeminine'},
  {0x00AB, '«', 'guillemotleft'},
  {0x00AC, '¬', 'notsign'},
    --{0x00AD, , 'hyphen'},
  {0x00AE, '®', 'registered'},
  {0x00AF, '¯', 'macron', 'dead_macron'},
  {0x00B0, '°', 'degree'},
  {0x00B1, '±', 'plusminus'},
  {0x00B2, '²', 'twosuperior'},
  {0x00B3, '³', 'threesuperior'},
  {0x00B4, '´', 'acute', 'dead_acute'},
  {0x00B5, 'µ', 'mu'},
  {0x00B6, '¶', 'paragraph'},
  {0x00B7, '·', 'periodcentered'},
  {0x00B8, '¸', 'cedilla', 'dead_cedilla'},
  {0x00B9, '¹', 'onesuperior'},
  {0x00BA, 'º', 'masculine'},
  {0x00BB, '»', 'guillemotright'},
  {0x00BC, '¼', 'onequarter'},
  {0x00BD, '½', 'onehalf'},
  {0x00BE, '¾', 'threequarters'},
  {0x00BF, '¿', 'questiondown'},
  {0x00C0, 'À', 'Agrave'},
  {0x00C1, 'Á', 'Aacute'},
  {0x00C2, 'Â', 'Acircumflex'},
  {0x00C3, 'Ã', 'Atilde'},
  {0x00C4, 'Ä', 'Adiaeresis'},
  {0x00C5, 'Å', 'Aring'},
  {0x00C6, 'Æ', 'AE'},
  {0x00C7, 'Ç', 'Ccedilla'},
  {0x00C8, 'È', 'Egrave'},
  {0x00C9, 'É', 'Eacute'},
  {0x00CA, 'Ê', 'Ecircumflex'},
  {0x00CB, 'Ë', 'Ediaeresis'},
  {0x00CC, 'Ì', 'Igrave'},
  {0x00CD, 'Í', 'Iacute'},
  {0x00CE, 'Î', 'Icircumflex'},
  {0x00CF, 'Ï', 'Idiaeresis'},
  {0x00D0, 'Ð', 'ETH'},
  {0x00D1, 'Ñ', 'Ntilde'},
  {0x00D2, 'Ò', 'Ograve'},
  {0x00D3, 'Ó', 'Oacute'},
  {0x00D4, 'Ô', 'Ocircumflex'},
  {0x00D5, 'Õ', 'Otilde'},
  {0x00D6, 'Ö', 'Odiaeresis'},
  {0x00D7, '×', 'multiply'},
  {0x00D8, 'Ø', 'Oslash'},
  {0x00D9, 'Ù', 'Ugrave'},
  {0x00DA, 'Ú', 'Uacute'},
  {0x00DB, 'Û', 'Ucircumflex'},
  {0x00DC, 'Ü', 'Udiaeresis'},
  {0x00DD, 'Ý', 'Yacute'},
  {0x00DE, 'Þ', 'THORN'},
  {0x00DF, 'ß', 'ssharp'},
  {0x00E0, 'à', 'agrave'},
  {0x00E1, 'á', 'aacute'},
  {0x00E2, 'â', 'acircumflex'},
  {0x00E3, 'ã', 'atilde'},
  {0x00E4, 'ä', 'adiaeresis'},
  {0x00E5, 'å', 'aring'},
  {0x00E6, 'æ', 'ae'},
  {0x00E7, 'ç', 'ccedilla'},
  {0x00E8, 'è', 'egrave'},
  {0x00E9, 'é', 'eacute'},
  {0x00EA, 'ê', 'ecircumflex'},
  {0x00EB, 'ë', 'ediaeresis'},
  {0x00EC, 'ì', 'igrave'},
  {0x00ED, 'í', 'iacute'},
  {0x00EE, 'î', 'icircumflex'},
  {0x00EF, 'ï', 'idiaeresis'},
  {0x00F0, 'ð', 'eth'},
  {0x00F1, 'ñ', 'ntilde'},
  {0x00F2, 'ò', 'ograve'},
  {0x00F3, 'ó', 'oacute'},
  {0x00F4, 'ô', 'ocircumflex'},
  {0x00F5, 'õ', 'otilde'},
  {0x00F6, 'ö', 'odiaeresis'},
  {0x00F7, '÷', 'division'},
  {0x00F8, 'ø', 'oslash'},
  {0x00F9, 'ù', 'ugrave'},
  {0x00FA, 'ú', 'uacute'},
  {0x00FB, 'û', 'ucircumflex'},
  {0x00FC, 'ü', 'udiaeresis'},
  {0x00FD, 'ý', 'yacute'},
  {0x00FE, 'þ', 'thorn'},
  {0x00FF, 'ÿ', 'ydiaeresis'},

  -- Latin Extented-A
  {0x0100, 'Ā', 'Amacron'},
  {0x0101, 'ā', 'amacron'},
  {0x0102, 'Ă', 'Abreve'},
  {0x0103, 'ă', 'abreve'},
  {0x0104, 'Ą', 'Aogonek'},
  {0x0105, 'ą', 'aogonek'},
  {0x0106, 'Ć', 'Cacute'},
  {0x0107, 'ć', 'cacute'},
  {0x0108, 'Ĉ', 'Ccircumflex'},
  {0x0109, 'ĉ', 'ccircumflex'},
  {0x010A, 'Ċ', 'Cabovedot'},
  {0x010B, 'ċ', 'cabovedot'},
  {0x010C, 'Č', 'Ccaron'},
  {0x010D, 'č', 'ccaron'},
  {0x010E, 'Ď', 'Dcaron'},
  {0x010F, 'ď', 'dcaron'},
  {0x0110, 'Đ', 'Dstroke'},
  {0x0111, 'đ', 'dstroke'},
  {0x0112, 'Ē', 'Emacron'},
  {0x0113, 'ē', 'emacron'},
    -- {0x0114, 'Ĕ', },
    -- {0x0115, 'ĕ', },
  {0x0116, 'Ė', 'Eabovedot'},
  {0x0117, 'ė', 'eabovedot'},
  {0x0118, 'Ę', 'Eogonek'},
  {0x0119, 'ę', 'eogonek'},
  {0x011A, 'Ě', 'Ecaron'},
  {0x011B, 'ě', 'ecaron'},
  {0x011C, 'Ĝ', 'Gcircumflex'},
  {0x011D, 'ĝ', 'gcircumflex'},
  {0x011E, 'Ğ', 'Gcaron'},
  {0x011F, 'ğ', 'gcaron'},
  {0x0120, 'Ġ', 'Gabovedot'},
  {0x0121, 'ġ', 'gabovedot'},
  {0x0122, 'Ģ', 'Gcedilla'},
  {0x0123, 'ģ', 'gcedilla'},
  {0x0124, 'Ĥ', 'Hcircumflex'},
  {0x0125, 'ĥ', 'hcircumflex'},
  {0x0126, 'Ħ', 'Hstroke'},
  {0x0127, 'ħ', 'hstroke'},
  {0x0128, 'Ĩ', 'Itilde'},
  {0x0129, 'ĩ', 'itilde'},
  {0x012A, 'Ī', 'Imacron'},
  {0x012B, 'ī', 'imacron'},
  {0x012C, 'Ĭ', 'Ibreve'},
  {0x012D, 'ĭ', 'ibreve'},
  {0x012E, 'Į', 'Iogonek'},
  {0x012F, 'į', 'iogonek'},
  {0x0130, 'İ', 'Iabovedot'},
  {0x0131, 'ı', 'idotless'},
    -- {0x0132, 'Ĳ', },
    -- {0x0133, 'ĳ', ''},
  {0x0134, 'Ĵ', 'Jcircumflex'},
  {0x0135, 'ĵ', 'jcircumflex'},
  {0x0136, 'Ķ', 'Kcedilla'},
  {0x0137, 'ķ', 'kcedilla'},
  {0x0138, 'ĸ', 'kra'},
  {0x0139, 'Ĺ', 'Lacute'},
  {0x013A, 'ĺ', 'lacute'},
  {0x013B, 'Ļ', 'Lcedilla'},
  {0x013C, 'ļ', 'lcedilla'},
  {0x013D, 'Ľ', 'Lcaron'},
  {0x013E, 'ľ', 'lcaron'},
    -- {0x013F, 'Ŀ',},
    -- {0x0140, 'ŀ',},
  {0x0141, 'Ł', 'Lstroke'},
  {0x0142, 'ł', 'lstroke'},
  {0x0143, 'Ń', 'Nacute'},
  {0x0144, 'ń', 'nacute'},
  {0x0145, 'Ņ', 'Ncedilla'},
  {0x0146, 'ņ', 'ncedilla'},
  {0x0147, 'Ň', 'Ncaron'},
  {0x0148, 'ň', 'ncaron'},
    -- {0x0149, 'ŉ',},
  {0x014A, 'Ŋ', 'ENG'},
  {0x014B, 'ŋ', 'eng'},
  {0x014C, 'Ō', 'Omacron'},
  {0x014D, 'ō', 'omacron'},
    -- {0x014E, 'Ŏ', ''},
    -- {0x014F, 'ŏ', ''},
  {0x0150, 'Ő', 'Odoubleacute'},
  {0x0151, 'ő', 'odoubleacute'},
  {0x0152, 'Œ', 'OE'},
  {0x0153, 'œ', 'oe'},
  {0x0154, 'Ŕ', 'Racute'},
  {0x0155, 'ŕ', 'racute'},
  {0x0156, 'Ŗ', 'Rcedilla'},
  {0x0157, 'ŗ', 'rcedilla'},
  {0x0158, 'Ř', 'Rcaron'},
  {0x0159, 'ř', 'rcaron'},
  {0x015A, 'Ś', 'Sacute'},
  {0x015B, 'ś', 'sacute'},
  {0x015C, 'Ŝ', 'Scircumflex'},
  {0x015D, 'ŝ', 'scircumflex'},
  {0x015E, 'Ş', 'Scedilla'},
  {0x015F, 'ş', 'scedilla'},
  {0x0160, 'Š', 'Scaron'},
  {0x0161, 'š', 'scaron'},
  {0x0162, 'Ţ', 'Tcedilla'},
  {0x0163, 'ţ', 'tcedilla'},
  {0x0164, 'Ť', 'Tcaron'},
  {0x0165, 'ť', 'tcaron'},
  {0x0166, 'Ŧ', 'Tslash'},
  {0x0167, 'ŧ', 'tslash'},
  {0x0168, 'Ũ', 'Utilde'},
  {0x0169, 'ũ', 'utilde'},
  {0x016A, 'Ū', 'Umacron'},
  {0x016B, 'ū', 'umacron'},
  {0x016C, 'Ŭ', 'Ubreve'},
  {0x016D, 'ŭ', 'ubreve'},
  {0x016E, 'Ů', 'Uring'},
  {0x016F, 'ů', 'uring'},
  {0x0170, 'Ű', 'Udoubleacute'},
  {0x0171, 'ű', 'udoubleacute'},
  {0x0172, 'Ų', 'Uogonek'},
  {0x0173, 'ų', 'uogonek'},
  {0x0174, 'Ŵ', 'Wcircumflex'},
  {0x0175, 'ŵ', 'wcircumflex'},
  {0x0176, 'Ŷ', 'Ycircumflex'},
  {0x0177, 'ŷ', 'ycircumflex'},
  {0x0178, 'Ÿ', 'Ydiaeresis'},
  {0x0179, 'Ź', 'Zacute'},
  {0x017A, 'ź', 'zacute'},
  {0x017B, 'Ż', 'Zabovedot'},
  {0x017C, 'ż', 'zabovedot'},
  {0x017D, 'Ž', 'Zcaron'},
  {0x017E, 'ž', 'zcaron'},
    -- {0x017F, 'ſ',},
})
